# -*- coding: utf-8 -*-

"""Basic implementation of an advertisement."""

from __future__ import annotations

__all__ = ['CompositeAdvertisementWidget']

import typing
import warnings

from qtpy import QtCore
from qtpy import QtWidgets
from anaconda_navigator.utils import url_utils
from anaconda_navigator import widgets
from . import core
from . import resources

if typing.TYPE_CHECKING:
    import typing_extensions

    class SimpleAdvertisementDict(typing_extensions.TypedDict):
        """Definition of a single simple advertisement."""

        text: str
        redirect_url: str
        image_url: str

    class CompositeAdvertisementDict(core.WidgetDict):  # pylint: disable=too-few-public-methods
        """Collection of a simple advertisements, that might be rotated in a single location."""

        advertisements: typing.MutableSequence[SimpleAdvertisementDict]


SLIDESHOW_TIMEOUT: 'typing_extensions.Final[int]' = 6000


class CustomStackedWidget(QtWidgets.QStackedWidget):
    """Widget for carousel of advertisements."""

    def __init__(self, parent: CompositeAdvertisementWidget) -> None:
        """Initialize new :class:`~CustomStackedWidget` instance."""
        super().__init__()

        advertisement: 'SimpleAdvertisementDict'
        for advertisement in parent.content:
            url: typing.Optional[str] = advertisement.get('redirect_url', None)
            if not url:
                warnings.warn('no `redirect_url` provided for an advertisement')
                continue

            image: typing.Optional[bytes] = None
            image_url: typing.Optional[str] = advertisement.get('image_url', None)
            if image_url:
                image = resources.load_resource(image_url)

            text: typing.Optional[str] = advertisement.get('text', None)
            if (not text) and (not image):
                warnings.warn('`text` or valid `image_url` should be provided for an advertisement')
                continue

            widget: widgets.LabelImageLinkVertical = widgets.LabelImageLinkVertical(
                text=text,
                url=parent.settings.inject_url_parameters(
                    url,
                    utm_medium='ad',
                    utm_content=url_utils.file_name(image_url),
                ),
                image=image,
                width=150,  # NOTE: width-height-margin should be calculated from placement
                height=150,
                margin=10,
            )
            widget.setFocusPolicy(QtCore.Qt.StrongFocus)
            self.addWidget(widget)

        self.__timer: 'typing_extensions.Final[QtCore.QTimer]' = QtCore.QTimer(self)
        self.__timer.setInterval(SLIDESHOW_TIMEOUT)
        self.__timer.timeout.connect(self.next)
        if self.count() > 1:
            self.__timer.start()

    def next(self) -> None:
        """Show next advertisement."""
        current: int = self.currentIndex()
        if current < 0:
            return
        current += 1
        if current >= self.count():
            current = 0
        self.setCurrentIndex(current)


CONTENT = typing.Sequence['SimpleAdvertisementDict']


class CompositeAdvertisementWidget(core.PartnerWidget[CustomStackedWidget]):
    """Widget with a simple image advertisement banners."""

    __slots__ = ('__content',)

    def __init__(self, definition: 'CompositeAdvertisementDict', settings: core.PartnerSettings) -> None:
        """Initialize new :class:`~PartnerWidget` instance."""
        super().__init__(
            settings=settings,
            placement=definition['placement'],
        )

        self.__content: 'typing_extensions.Final[CONTENT]' = definition['advertisements']

    @property
    def content(self) -> CONTENT:  # noqa: D401
        """Description of the advertisements to show in widget."""
        return self.__content

    def widget(self) -> CustomStackedWidget:
        """Generate new widget to use in Qt application."""
        return CustomStackedWidget(parent=self)



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Constants used by package manager widget."""

from collections import OrderedDict
import enum
import gettext


_ = gettext.gettext

# Constants
COLUMNS = (
    COL_START,
    COL_ACTION,
    COL_NAME,
    COL_PACKAGE_TYPE,
    COL_DESCRIPTION,
    COL_VERSION,
    COL_STATUS,
    COL_URL,
    COL_ACTION_VERSION,
    COL_END,

    COL_UPGRADE,
    COL_INSTALL,
    COL_REMOVE,
    COL_DOWNGRADE,
) = list(range(0, 14))

ACTION_COLUMNS = [
    COL_ACTION,
    COL_ACTION_VERSION,
]

ACTIONS = (
    ACTION_NONE,
    ACTION_INSTALL,
    ACTION_REMOVE,
    ACTION_UPGRADE,
    ACTION_DOWNGRADE,
    ACTION_UPDATE,  # equivalent to conda update, no version specified
    ACTION_CREATE,
    ACTION_CLONE,
    ACTION_REMOVE_ENV,
    ACTION_IMPORT,
    ACTION_SEARCH,
    ACTION_EXPORT,
) = list(range(100, 112))

PACKAGE_TYPES = (
    CONDA_PACKAGE,
    PIP_PACKAGE,
) = ['     conda', '    pip']

PACKAGE_STATUS = (
    INSTALLED,
    NOT_INSTALLED,
    UPGRADABLE,
    DOWNGRADABLE,
    SELECTED,
    ALL,
    MIXGRADABLE,
) = list(range(200, 207))

COMBOBOX_VALUES_ORDERED = [
    'Installed',
    'Not installed',
    'Updatable',
    'Selected',
    'All',
]

COMBO_PACKAGE_STATUS = PACKAGE_STATUS[:]
COMBO_PACKAGE_STATUS.remove(DOWNGRADABLE)
COMBO_PACKAGE_STATUS.remove(MIXGRADABLE)
COMBOBOX_VALUES = OrderedDict(zip(COMBOBOX_VALUES_ORDERED, COMBO_PACKAGE_STATUS))
ROOT = 'root'
UPGRADE_SYMBOL = ' ⬆'

# Application actions
APPLICATION_INSTALL = 'install'
APPLICATION_UPDATE = 'update'
APPLICATION_REMOVE = 'remove'
APPLICATION_LAUNCH = 'launch'

# Widget names for senders
MAIN_UPDATE = 'main-update'
ENVIRONMENT_PACKAGE_MANAGER = 'environment-package-manager'
TAB_HOME = 'tab-home'
TAB_ENVIRONMENT = 'tab-environment'
TAB_LEARNING = 'tab-learning'
TAB_COMMUNITY = 'tab-community'

ACTION_2_WORD = {
    ACTION_NONE: 'No action',
    ACTION_INSTALL: 'Installing',
    ACTION_REMOVE: 'Removing',
    ACTION_UPGRADE: 'Updating',
    ACTION_DOWNGRADE: 'Downgrading',
    ACTION_CREATE: 'Creating',
    ACTION_CLONE: 'Cloning',
    ACTION_REMOVE_ENV: 'Removing',
    ACTION_IMPORT: 'Importing',
    ACTION_SEARCH: 'Searching',
}


class AppType(str, enum.Enum):
    """Options for the application types."""

    WEB = 'web_app'
    CONDA = 'conda_app'
    INSTALLABLE = 'installable'
